<?php
/*
* Copyright (c) 2005 Heiko Rutenbeck <bzrudi@tuxpower.de>
*                    Florian Angehrn 
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**
 * This class holds a number of methods for filesys related access
 * @package Filesystem
 */


class LinFilesys
{


/**
* create directory recursively - same as "mkdir -p"
*/
function mkdir_p($target,$mode='0700')
{
	if(is_dir($target) || empty($target)) {	// best case check first
		return 1;
	}
	if(file_exists($target) && !is_dir($target)) {	// target exists bug isn't a directory..
		return 0;
	}
	if( mkdir_p( substr( $target,0,strrpos( $target,'/') ) ) )
	{
		return mkdir($target,intval($mode,8)); // crawl back up & create dir tree
	}
	return 0;
}

/**
 * delete recurisvely - same as "rm -rf"
 * @todo  should we really have such a dangerous function in linpha?
 * @param string $path folder to be deleted
 * @param boolean $followLinks delete also contents inside links
 */
function rm_rf($path,$followLinks=false)
{
	if(empty($path)) {
		return false;
	}
	
	if( is_file($path) )
	{
		@unlink( $path );
		return true;
	}
	else
	{
		$dir = opendir($path) ;
		while (false !== ($entry = readdir($dir)))
		{
			if ( is_file( "$path/$entry" ) || ((!$followLinks) && is_link("$path/$entry")) )
			{
				@unlink( "$path/$entry" );
			}
			elseif ( is_dir( "$path/$entry" ) && $entry!='.' && $entry!='..' )
			{
				LinFilesys::rm_rf( "$path/$entry" );
			}
		}
		closedir($dir) ;
		return @rmdir($path);
	}
}

/**
 * realpath() doesnt work somtimes if path is absolute
 * linRealpath takes care of this
 */
function linRealpath( $path)
{
	if( ! LinFilesys::isAbsolutePath( $path ) )
	{
		$path = realpath( $path );
	}
	
	/**
	 * remove ending '/' if exists
	 */
	$len = strlen( $path );
	$lastsign = substr( $path, $len-1);
	if($lastsign == '/' OR $lastsign == '\\')
	{
		$path = substr($path,0,$len-1);
	}
	
	return $path;
}

/**
* returns a unique filename, but different to tempnam() the file doesn't exists
*
* @author  flo
* @param  string  $prefix  prefix for the tmp_file
* @param  string  $suffix  suffix for the tmp_file, formerly the file extension
* @return  string  unique temporary filename
* @package  filesys
*
* http://ch.php.net/manual/de/function.uniqid.php
*/
function linTempnam($prefix,$suffix,$use_absolutepath=false)
{
	$tmp_dir = LinSql::getPath('tmp');
	
	if($use_absolutepath) // used in archiver.class.php because of chdir()
	{
		$tmp_dir = LinFilesys::linRealpath($tmp_dir);
	}
	
	if(!is_writable($tmp_dir)) {
		error_log("linpha_tempnam: tmp_dir ($tmp_dir) isn't writable! Using /tmp instead");
		$tmp_dir = '/tmp';
	}
	
	for($i=0;  file_exists($tmp_dir.'/'.$prefix.'_'.session_id().'_'.$i.$suffix);  $i++) { }
	$tmp_file = $tmp_dir.'/'.$prefix.'_'.session_id().'_'.$i.$suffix;
	touch($tmp_file);

	return $tmp_file;
}

/**
 * create a unique tmp folder
 */
function createTmpFolder($prefix)
{
	$tmp_dir = $GLOBALS['linpha']->sql->config->value['sys_path_tmp_dir'];
	for($i=0;  file_exists(LinFilesys::getFullPath($tmp_dir).'/'.$prefix.'_'.session_id().'_'.$i);  $i++) { }
	$tmp_folder = LinFilesys::getFullPath($tmp_dir).'/'.$prefix.'_'.session_id().'_'.$i;
	mkdir($tmp_folder,0700);
	return $tmp_folder;
}

/**
 * close session before exec()
 * if exec() is called twice (like on the print preview), apache will hang
 * on windows
 * 
 * $_SESSION is still available after session_write_close!
 * but we cannot edit it!!!!
 */
function linExec($str, &$array_output=array(), &$return_value='')
{
	session_write_close();

	exec( $str, $array_output, $return_value );
}


/**
* if $path is absolute, input is just returned, if $path is relative, 
* the relative path starting from the linpha root directory is returned
* 
* always removed ending '/' to have unified paths in linpha
*
* @author  flo
* @param  string  $path  a path
* @return  string  a path
* @package  filesys
*/
function getFullPath($path)
{
	if( ! LinFilesys::isAbsolutePath($path))
	{
		$path = LINPHA_DIR.'/'.$path;
	}
	
	/**
	 * remove ending '/' if exists
	 */
		$len = strlen( $path );
		$lastsign = substr( $path, $len-1);
		if($lastsign == '/' OR $lastsign == '\\')
		{
			$path = substr($path,0,$len-1);
		}
	
	return $path;
}


/**
* checks wether $path is absolute or relative
*
* @author  flo
* @param  string  $path  a path
* @return  true|false  if is an absolute path or not
* @package  filesys
*/
function isAbsolutePath($path)
{
	if(substr($path,0,1) == '/' OR		// string begins with a '/' -> absolut path
		((strpos($path, ':') !== false) AND (strpos( PHP_OS, 'WIN' ) !== FALSE))	// we are on windows and the string contains a ':' -> absolut path (c:/..)
		) {	// absolut path
		return true;
	} else {
		return false;
	}
}


/**
* gets the file extension from a path/filename.ext, filename.ext, .ext or ext* 
*
* @author  flo
* @param  string  $filename  path to file
* @return  string  file extension
* @package  filesys
*/
function getFileExtFromPath($filename)
{
	if(substr_count($filename,'.') == 0) {
		return $filename;	// no point found, -> is already the extension
	}
	$pos = strrpos( $filename, '.' );	// find last point
	$ext = substr( $filename, $pos+1, strlen( $filename ) );	// extract extension
	$ext = strtolower( $ext );	// convert to lowercase
	return $ext;
}

/**
 * gets the filename withouth the file extension (index.html > index)
 * 
 * @author  flo
 * @param  string  $filename
 * @return  string  filename without file extension
 * @package  filesys
 * @uses  admin/maintenance_db.php
 */
function getFilenameWithoutFileExt($filename)
{
	if(substr_count($filename,'.') == 0) {
		return $filename;	// no point found, -> is already filename without the extension
	}
	$pos = strrpos( $filename, '.' );	// find last point
	$filename = substr( $filename, 0, $pos );	// extract filename

	return $filename;
}

/**
 * This function formats the filesize (byte) value to kB/MB/GB
 */
function niceFilesize($filesize, $precision)
{
	if ($filesize < 1000) {
		$sizeunit  = 'Bytes';
		$precision = 0;
	} else {
		$filesize /= 1024;
		$sizeunit = 'KB';
	}
	if ($filesize >= 1000) {
		$filesize /= 1024;
		$sizeunit = 'MB';
	}
	if ($filesize >= 1000) {
		$filesize /= 1024;
		$sizeunit = 'GB';
	}
	return number_format($filesize, $precision).' '.$sizeunit;
}

/**
 * get a clean path
 * input:  ..//./../dir4//./dir5/dir6/..//dir7/
 * output: ../../dir4/dir5/dir7/
 * 
 * the php builtin function realpath does not work if the file does not exists
 * 
 * thanks to http://ch2.php.net/realpath
 */
function cleanPath($path) {
	$result = array();
	// $pathA = preg_split('/[\/\\\]/', $path);
	$pathA = explode('/', $path);
	if (!$pathA[0])
		$result[] = '';
	foreach ($pathA AS $key => $dir) {
		if ($dir == '..') {
			if (end($result) == '..') {
				$result[] = '..';
			} elseif (!array_pop($result)) {
				$result[] = '..';
			}
		} elseif ($dir && $dir != '.') {
			$result[] = $dir;
		}
	}
	if (!end($pathA))
		$result[] = '';
	return implode('/', $result);
}

/**
 * copy a file with a timestamp
 * 
 * @author  flo
 * @param  $filename  string  path to file to backup
 * @return  bool  true if successful, false on failure
 * @uses  LinMetaDataModificate::saveModifiedMetaData()
 */
function backupFile($filename)
{
	if( !file_exists($filename))
	{
		return false;
	}
	
	$dirname = dirname($filename);
	$basename = basename($filename);
	
	$pos = strrpos($basename,'.');
	$file_without_ext = substr($basename,0,$pos);
	$file_ext = substr($basename,$pos+1);
	
	$timeformat = strftime("%Y%m%d_%H%M%S");
	
	if( @copy($filename, $dirname.'/.'.$file_without_ext.'_bak_'.$timeformat.'.'.$file_ext) )
	{
		return true;
	}
	else
	{
		return false;
	}
}

} // end class LinFile
?>
